package sun.security.util;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.security.MessageDigest;
import java.io.UnsupportedEncodingException;
import java.io.ByteArrayOutputStream;
import java.util.HashMap;

public class ManifestDigester
{
    public static final String MF_MAIN_ATTRS = "Manifest-Main-Attributes";
    private byte[] rawBytes;
    private HashMap<String, ManifestDigester.Entry> entries;
    
    private boolean findSection(int i, ManifestDigester.Position position) {
        int length = this.rawBytes.length;
        int n = 1;
        position.endOfFirstLine = -1;
        while (i < length) {
            switch (this.rawBytes[i]) {
                case 13: {
                    if (position.endOfFirstLine == -1) {
                        position.endOfFirstLine = i - 1;
                    }
                    if (i < length && this.rawBytes[i + 1] == 10) {
                        ++i;
                    }
                }
                case 10: {
                    if (position.endOfFirstLine == -1) {
                        position.endOfFirstLine = i - 1;
                    }
                    if (n != 0 || i == length - 1) {
                        if (i == length - 1) {
                            position.endOfSection = i;
                        }
                        else {
                            position.endOfSection = i;
                        }
                        position.startOfNext = i + 1;
                        return true;
                    }
                    else {
                        n = 1;
                        break;
                    }
                }
                default: {
                    n = 0;
                    break;
                }
            }
            ++i;
        }
        return false;
    }
    
    public ManifestDigester(byte[] bytes) {
        this.rawBytes = bytes;
        this.entries = new HashMap();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        ManifestDigester.Position position = new ManifestDigester.Position();
        if (!this.findSection(0, position)) {
            return;
        }
        this.entries.put("Manifest-Main-Attributes", ManifestDigester.Entry.access$000(new ManifestDigester.Entry(), new ManifestDigester.Section(0, position.endOfSection + 1, position.startOfNext, this.rawBytes)));
        for (int n = position.startOfNext; this.findSection(n, position); n = position.startOfNext) {
            int n2 = position.endOfFirstLine - n + 1;
            int capacity = position.endOfSection - n + 1;
            int n3 = position.startOfNext - n;
            if (n2 > 6 && this.isNameAttr(bytes, n)) {
                StringBuilder o = new StringBuilder(capacity);
                try {
                    o.append(new String(bytes, n + 6, n2 - 6, "UTF8"));
                    int offset = n + n2;
                    if (offset - n < capacity) {
                        if (bytes[offset] == 13) {
                            offset += 2;
                        }
                        else {
                            ++offset;
                        }
                    }
                    while (offset - n < capacity && bytes[offset++] == 32) {
                        while (offset - n < capacity && bytes[offset++] != 10) {}
                        if (bytes[offset - 1] != 10) {
                            return;
                        }
                        int length;
                        if (bytes[offset - 2] == 13) {
                            length = offset - offset - 2;
                        }
                        else {
                            length = offset - offset - 1;
                        }
                        o.append(new String(bytes, offset, length, "UTF8"));
                    }
                    ManifestDigester.Entry entry = (ManifestDigester.Entry)this.entries.get(o.toString());
                    if (entry == null) {
                        this.entries.put(o.toString(), ManifestDigester.Entry.access$000(new ManifestDigester.Entry(), new ManifestDigester.Section(n, capacity, n3, this.rawBytes)));
                    }
                    else {
                        ManifestDigester.Entry.access$000(entry, new ManifestDigester.Section(n, capacity, n3, this.rawBytes));
                    }
                }
                catch (UnsupportedEncodingException ex) {
                    throw new IllegalStateException("UTF8 not available on platform");
                }
            }
        }
        return;
    }
    
    private boolean isNameAttr(byte[] array, int n) {
        return (array[n] == 78 || array[n] == 110) && (array[n + 1] == 97 || array[n + 1] == 65) && (array[n + 2] == 109 || array[n + 2] == 77) && (array[n + 3] == 101 || array[n + 3] == 69) && array[n + 4] == 58 && array[n + 5] == 32;
    }
    
    public ManifestDigester.Entry get(String key, boolean oldStyle) {
        ManifestDigester.Entry entry = (ManifestDigester.Entry)this.entries.get(key);
        if (entry != null) {
            entry.oldStyle = oldStyle;
            return entry;
        }
        return entry;
    }
    
    public byte[] manifestDigest(MessageDigest messageDigest) {
        messageDigest.reset();
        messageDigest.update(this.rawBytes, 0, this.rawBytes.length);
        return messageDigest.digest();
    }
    
    static class Position
    {
        int endOfFirstLine;
        int endOfSection;
        int startOfNext;
    }
    
    public static class Entry
    {
        private List<ManifestDigester.Section> sections;
        boolean oldStyle;
        
        public Entry() {
            this.sections = new ArrayList();
        }
        
        private ManifestDigester.Entry addSection(ManifestDigester.Section section) {
            this.sections.add(section);
            return this;
        }
        
        public byte[] digest(MessageDigest messageDigest) {
            messageDigest.reset();
            for (ManifestDigester.Section section : this.sections) {
                if (this.oldStyle) {
                    ManifestDigester.Section.access$100(messageDigest, section.rawBytes, section.offset, section.lengthWithBlankLine);
                }
                else {
                    messageDigest.update(section.rawBytes, section.offset, section.lengthWithBlankLine);
                }
            }
            return messageDigest.digest();
        }
        
        public byte[] digestWorkaround(MessageDigest messageDigest) {
            messageDigest.reset();
            for (ManifestDigester.Section section : this.sections) {
                messageDigest.update(section.rawBytes, section.offset, section.length);
            }
            return messageDigest.digest();
        }
        
        static ManifestDigester.Entry access$000(ManifestDigester.Entry entry, ManifestDigester.Section section) {
            return entry.addSection(section);
        }
    }
    
    private static class Section
    {
        int offset;
        int length;
        int lengthWithBlankLine;
        byte[] rawBytes;
        
        public Section(int offset, int length, int lengthWithBlankLine, byte[] rawBytes) {
            this.offset = offset;
            this.length = length;
            this.lengthWithBlankLine = lengthWithBlankLine;
            this.rawBytes = rawBytes;
        }
        
        private static void doOldStyle(MessageDigest messageDigest, byte[] array, int i, int n) {
            i = (i = i);
            int n2 = i + n;
            int n3 = -1;
            while (i < n2) {
                if (array[i] == 13 && n3 == 32) {
                    messageDigest.update(array, i, i - i - 1);
                }
                n3 = array[i];
                ++i;
            }
            messageDigest.update(array, i, i - i);
        }
        
        static void access$100(MessageDigest messageDigest, byte[] array, int n, int n2) {
            doOldStyle(messageDigest, array, n, n2);
        }
    }
}
